# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
# --------------------------------------
# Copyright (C) 2006-2009 Nicolas Schulz
#               2008-2009 Florian Noeding (Python wrapper)
#               2009 Alex Fuller (Python wrapper update)
#
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# *************************************************************************************************

# Updated to Horde3D version beta 4 (community SVN 331 to be exact - Alex)

from ctypes import *

if not 'c_bool' in globals():
	c_bool = c_int


__all__ = []

try:
	import utils
	__all__.append('utils')
except:
	pass

try:
	import terrain
	__all__.append('terrain')
except:
	pass

try:
	import sound
	__all__.append('sound')
except:
	pass


try:
	h3d = cdll.LoadLibrary('libHorde3D.so')
except OSError:
	h3d = cdll.LoadLibrary('Horde3D.dll')


H3DRootNode = 1
__all__.append('H3DRootNode')


class H3DOptions(object):
	MaxLogLevel = 1
	MaxNumMessages = 2
	TrilinearFiltering = 3
	MaxAnisotropy = 4
	TexCompression = 5
	LoadTextures = 6
	FastAnimation = 7
	ShadowMapSize = 8
	SampleCount = 9
	WireframeMode = 10
	DebugViewMode = 11
	DumpFailedShaders = 12
	GatherTimeStats = 13
__all__.append('H3DOptions')


class H3DStats(object):
	TriCount = 100
	BatchCount = 101
	LightPassCount = 102
	FrameTime = 103
	AnimationTime = 104
	CustomTime = 105
	TextureVMem = 106
	GeometryVMem = 107
__all__.append('H3DStats')


class H3DResTypes(object):
	Undefined = 0
	SceneGraph = 1
	Geometry = 2
	Animation = 3
	Material = 4
	Code = 5
	Shader = 6
	Texture = 7
	ParticleEffect = 8
	Pipeline = 9
__all__.append('H3DResTypes')


class H3DResFlags(object):
	NoQuery = 1
	NoTexCompression = 2
	NoTexMipmaps = 4
	TexCubemap = 8
	TexDynamic = 16
	TexRenderable = 32
__all__.append('H3DResFlags')


class H3DFormats(object):
        Unknown = 0
	TEX_BGRA8 = 1
	TEX_DXT1 = 2
	TEX_DXT3 = 3
	TEX_DXT5 = 4
	TEX_RGBA16F = 5
	TEX_RGBA32F = 6
__all__.append('H3DFormats')
	

class H3DGeoRes(object):
	GeometryElem = 200
	GeoIndexCountI = 201
	GeoVertexCountI = 202
	GeoIndices16I = 203
	GeoIndexStream = 204
	GeoVertPosStream = 205
	GeoVertTanStream = 206
	GeoVertStaticStream = 207
__all__.append('H3DGeoRes')


class H3DAnimRes(object):
	EntityElem = 300
	EntFrameCountI = 301
__all__.append('H3DAnimRes')


class H3DMatRes(object):
	MaterialElem = 400
	SamplerElem = 401
	UniformElem = 402
	MatClassStr = 403
	MatLinkI = 404
	MatShaderI = 405
	SampNameStr = 406
	SampTexResI = 407
	UnifNameStr = 408
	UnifValueF4 = 409
__all__.append('H3DMatRes')


class H3DShaderRes(object):
	ContextElem = 600
	SamplerElem = 601
	UniformElem = 602
	ContNameStr = 603
	SampNameStr = 604
	UnifNameStr = 605
	UnifDefValueF4 = 606
__all__.append('H3DShaderRes')

class H3DTexRes(object):
	TextureElem = 700
	ImageElem = 701
	TexFormatI = 702
	TexSliceCountI = 703
	ImgWidthI = 704
	ImgHeightI = 705
	ImgPixelStream = 706
__all__.append('H3DTexRes')


class H3DPartEffRes(object):
	ParticleElem = 800
	ChanMoveVelElem = 801
	ChanRotVelElem = 802
	ChanSizeElem = 803
	ChanColRElem = 804
	ChanColGElem = 805
	ChanColBElem = 806
	ChanColAElem = 807
	PartLifeMinF = 808
	PartLifeMaxF = 809
	ChanStartMinF = 810
	ChanStartMaxF = 811
	ChanEndRateF = 812
__all__.append('H3DPartEffRes')


class H3DPipeRes(object):
	StageElem = 900
	StageNameStr = 901
	StageActivationI = 902
__all__.append('H3DPipeRes')


class H3DNodeTypes(object):
	Undefined = 0
	Group = 1
	Model = 2
	Mesh = 3
	Joint = 4
	Light = 5
	Camera = 6
	Emitter = 7
__all__.append('H3DNodeTypes')


class H3DNodeParams(object):
	NameStr = 1
	AttachmentStr = 2
__all__.append('H3DNodeParams')


class H3DModel(object):
	GeoResI = 200
	SWSkinningI = 201
	LodDist1F = 202
	LodDist2F = 203
	LodDist3F = 204
	LodDist4F = 205
__all__.append('H3DModel')


class H3DMesh(object):
	MatResI = 300
	BatchStartI = 301
	BatchCountI = 302
	VertRStartI = 303
	VertREndI = 304
	LodLevelI = 305
__all__.append('H3DMesh')


class H3DJoint(object):
	JointIndexI = 400
__all__.append('H3DJoint')


class H3DLight(object):
	MatResI = 500
	RadiusF = 501
	FovF = 502
	ColorF3 = 503
	ShadowMapCountI = 504
	ShadowSplitLambdaF = 505
	ShadowMapBiasF = 506
	LightingContextStr = 507
	ShadowContextStr = 508
__all__.append('H3DLight')


class H3DCamera(object):
	PipeResI = 600
	OutTexResI = 601
	OutBufIndexI = 602
	LeftPlaneF = 603
	RightPlaneF = 604
	BottomPlaneF = 605
	TopPlaneF = 606
	NearPlaneF = 607
	FarPlaneF = 608
	OrthoI = 609
	OccCullingI = 610
__all__.append('H3DCamera')


class H3DEmitter(object):
	MatResI = 700
	PartEffResI = 701
	MaxCountI = 702
	RespawnCountI = 703
	DelayF = 704
	EmissionRateF = 705
	SpreadAngleF = 706
	ForceF3 = 707
__all__.append('H3DEmitter')


h3dGetVersionString = h3d.h3dGetVersionString
h3dGetVersionString.restype = c_char_p
h3dGetVersionString.argtypes = []
__all__.append('h3dGetVersionString')


h3dCheckExtension = h3d.h3dCheckExtension
h3dCheckExtension.restype = c_bool
h3dCheckExtension.argtypes = [c_char_p]
__all__.append('h3dCheckExtension')


h3dGetError = h3d.h3dGetError
h3dGetError.restype = c_bool
h3dGetError.argtypes = []
__all__.append('h3dGetError')


h3dInit = h3d.h3dInit
h3dInit.restype = c_bool
h3dInit.argtypes = []
__all__.append('h3dInit')


h3dRelease = h3d.h3dRelease
h3dRelease.restype = None
h3dRelease.argtypes = []
__all__.append('h3dRelease')


h3dSetupViewport = h3d.h3dSetupViewport
h3dSetupViewport.restype = None
h3dSetupViewport.argtypes = [c_int, c_int, c_int, c_int, c_bool]
__all__.append('h3dSetupViewport')


h3dRender = h3d.h3dRender
h3dRender.restype = None
h3dRender.argtypes = [c_int]
__all__.append('h3dRender')


h3dFinalizeFrame = h3d.h3dFinalizeFrame
h3dFinalizeFrame.restype = None
h3dFinalizeFrame.argtypes = []
__all__.append('h3dFinalizeFrame')


h3dClear = h3d.h3dClear
h3dClear.restype = None
h3dClear.argtypes = []
__all__.append('h3dClear')


_h3dGetMessage = h3d.h3dGetMessage
_h3dGetMessage.restype = c_char_p
_h3dGetMessage.argtypes = [POINTER(c_int), POINTER(c_float)]
def h3dGetMessage():
	i = c_int()
	f = c_float()
	s = _h3dGetMessage(byref(i), byref(f))

	return (s, i.value, f.value)
__all__.append('h3dGetMessage')


h3dGetOption = h3d.h3dGetOption
h3dGetOption.restype = c_float
h3dGetOption.argtypes = [c_int]
__all__.append('h3dGetOption')


_h3dSetOption = h3d.h3dSetOption
_h3dSetOption.restype = c_bool
_h3dSetOption.argtypes = [c_int, c_float]
def h3dSetOption(param, value):
	return _h3dSetOption(param, c_float(value))
__all__.append('h3dSetOption')


_h3dGetStat = h3d.h3dGetStat
_h3dGetStat.restype = c_float
_h3dGetStat.argtypes = [c_int, c_bool]
def h3dGetStat(param, reset):
	return _h3dGetStat(param, reset).value
__all__.append('h3dGetStat')


_h3dShowOverlay = h3d.h3dShowOverlay
_h3dShowOverlay.restype = None
_h3dShowOverlay.argtypes = [
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_float, c_float, c_float, c_float,
		c_int, c_int]
def h3dShowOverlay(
		x_tl, y_tl, u_tl, v_tl,
		x_bl, y_bl, u_bl, v_bl,
		x_br, y_br, u_br, v_br,
		x_tr, y_tr, u_tr, v_tr,
		r, g, b, a,
		materialRes, layer):
	return _h3dShowOverlay(
			c_float(x_tl), c_float(y_tl), c_float(u_tl), c_float(v_tl),
			c_float(x_bl), c_float(y_bl), c_float(u_bl), c_float(v_bl),
			c_float(x_br), c_float(y_br), c_float(u_br), c_float(v_br),
			c_float(x_tr), c_float(y_tr), c_float(u_tr), c_float(v_tr),
			c_float(r), c_float(g), c_float(b), c_float(a),
			materialRes, layer
			)
__all__.append('h3dShowOverlay')


h3dClearOverlays = h3d.h3dClearOverlays
h3dClearOverlays.restype = None
h3dClearOverlays.argtypes = []
__all__.append('h3dClearOverlays')


h3dGetResType = h3d.h3dGetResType
h3dGetResType.restype = c_int
h3dGetResType.argtypes = [c_int]
__all__.append('h3dGetResType')


h3dGetResName = h3d.h3dGetResName
h3dGetResName.restype = c_char_p
h3dGetResName.argtypes = [c_int]
__all__.append('h3dGetResName')


h3dGetNextResource = h3d.h3dGetNextResource
h3dGetNextResource.restype = c_int
h3dGetNextResource.append = [c_int, c_int]
__all__.append('h3dGetNextResource')


h3dFindResource = h3d.h3dFindResource
h3dFindResource.restype = c_int
h3dFindResource.argtypes = [c_int, c_char_p]
__all__.append('h3dFindResource')


h3dAddResource = h3d.h3dAddResource
h3dAddResource.restype = c_int
h3dAddResource.argtypes = [c_int, c_char_p, c_int]
__all__.append('h3dAddResource')


h3dCloneResource = h3d.h3dCloneResource
h3dCloneResource.restype = c_int
h3dCloneResource.argtypes = [c_int, c_char_p]
__all__.append('h3dCloneResource')


h3dRemoveResource = h3d.h3dRemoveResource
h3dRemoveResource.restype = c_int
h3dRemoveResource.argtypes = [c_int]
__all__.append('h3dRemoveResource')


h3dIsResLoaded = h3d.h3dIsResLoaded
h3dIsResLoaded.restype = c_bool
h3dIsResLoaded.argtypes = [c_int]
__all__.append('h3dIsResLoaded')


## data should be c_char_p and not c_void_p?
_h3dLoadResource = h3d.h3dLoadResource
_h3dLoadResource.restype = c_bool
_h3dLoadResource.argtypes = [c_int, c_void_p, c_int]
def h3dLoadResource(res, data):
	return _h3dLoadResource(res, data, len(data))
__all__.append('h3dLoadResource')


h3dUnloadResource = h3d.h3dUnloadResource
h3dUnloadResource.restype = None
h3dUnloadResource.argtypes = [c_int]
__all__.append('h3dUnloadResource')

# h3dGetResElemCount - incomplete
h3dGetResElemCount = h3d.h3dGetResElemCount
h3dGetResElemCount.restype = c_int
h3dGetResElemCount.argtypes = [c_int, c_int]
__all__.append('h3dGetResElemCount')

# h3dFindResElem - incomplete
h3dFindResElem = h3d.h3dFindResElem
h3dFindResElem.restype = c_int
h3dFindResElem.argtypes = [c_int, c_int, c_int, c_char_p]
__all__.append('h3dFindResElem')

# h3dGetResParamI - incomplete
h3dGetResParamI = h3d.h3dGetResParamI
h3dGetResParamI.restype = c_int
h3dGetResParamI.argtypes = [c_int, c_int, c_int, c_int]
__all__.append('h3dGetResParamI')

# h3dSetResParamI - incomplete
h3dSetResParamI = h3d.h3dSetResParamI
h3dSetResParamI.restype = None
h3dSetResParamI.argtypes = [c_int, c_int, c_int, c_int, c_int]
__all__.append('h3dSetResParamI')

# h3dGetResParamF - It's completely WRONG!
_h3dGetResParamF = h3d.h3dGetResParamF
_h3dGetResParamF.restype = c_float
_h3dGetResParamF.argtypes = [c_int, c_int, c_int, c_int, c_int]
def h3dGetResParamF(res, elem, elemIdx, param, compIdx):
	return _h3dGetResParamF(res, elem, elemIdx, param, compIdx).value
__all__.append('h3dGetResParamF')

# h3dSetResParamF - incomplete
h3dSetResParamF = h3d.h3dSetResParamF
h3dSetResParamF.restype = None
h3dSetResParamF.argtypes = [c_int, c_int, c_int, c_int, c_int, c_float]
__all__.append('h3dSetResParamF')


h3dGetResParamStr = h3d.h3dGetResParamStr
h3dGetResParamStr.restype = c_char_p
h3dGetResParamStr.argtypes = [c_int, c_int, c_int, c_int]
__all__.append('h3dGetResParamStr')


h3dSetResParamStr = h3d.h3dSetResParamStr
h3dSetResParamStr.restype = None
h3dSetResParamStr.argtypes = [c_int, c_int, c_int, c_int, c_char_p]
__all__.append('h3dSetResParamStr')


## getResourceData - deprecated?
#getResourceData = h3d.getResourceData
#getResourceData.restype = c_void_p
#getResourceData.argtypes = [c_int, c_int]
#__all__.append('getResourceData')

## updateResourceData - deprecated?
#_updateResourceData = h3d.updateResourceData
#_updateResourceData.restype = c_bool
#_updateResourceData.argtypes = [c_int, c_int, c_void_p, c_int]
#def updateResourceData(res, param, data):
#	return _updateResourceData(res, param, data, len(data))
#__all__.append('updateResourceData')

## queryUnloadedResource - deprecated?
#queryUnloadedResource = h3d.queryUnloadedResource
#queryUnloadedResource.restype = c_int
#queryUnloadedResource.argtypes = [c_int]
#__all__.append('queryUnloadedResource')

## releaseUnusedResources - deprecated?
#releaseUnusedResources = h3d.releaseUnusedResources
#releaseUnusedResources.restype = None
#releaseUnusedResources.argtypes = []
#__all__.append('releaseUnusedResources')


# h3dMapResStream - incomplete?
h3dMapResStream = h3d.h3dMapResStream
h3dMapResStream.restype = c_void_p
h3dMapResStream.argtypes = [c_int, c_int, c_int, c_int, c_bool, c_bool]
__all__.append('h3dMapResStream')


h3dUnmapResStream = h3d.h3dUnmapResStream
h3dUnmapResStream.restype = None
h3dUnmapResStream.argtypes = [c_int]
__all__.append('h3dUnmapResStream')


h3dQueryUnloadedResource = h3d.h3dQueryUnloadedResource
h3dQueryUnloadedResource.restype = c_int
h3dQueryUnloadedResource.argtypes = [c_int]
__all__.append('h3dQueryUnloadedResource')


h3dReleaseUnusedResources = h3d.h3dReleaseUnusedResources
h3dReleaseUnusedResources.restype = None
h3dReleaseUnusedResources.argtypes = []
__all__.append('h3dReleaseUnusedResources')


h3dCreateTexture = h3d.h3dCreateTexture
h3dCreateTexture.restype = c_int
h3dCreateTexture.argtypes = [c_char_p, c_int, c_int, c_int, c_int]
__all__.append('h3dCreateTexture')


h3dSetShaderPreambles = h3d.h3dSetShaderPreambles
h3dSetShaderPreambles.restype = None
h3dSetShaderPreambles.argtypes = [c_char_p, c_char_p]
__all__.append('h3dSetShaderPreambles')


_h3dSetMaterialUniform = h3d.h3dSetMaterialUniform
_h3dSetMaterialUniform.restype = c_bool
_h3dSetMaterialUniform.argtypes = [c_int, c_char_p, c_float, c_float, c_float, c_float]
def h3dSetMaterialUniform(materialRes, name, a, b, c, d):
	return _h3dSetMaterialUniform(materialRes, name, c_float(a), c_float(b), c_float(c), c_float(d))
__all__.append('h3dSetMaterialUniform')

## setMaterialSampler - deprecated?
#setMaterialSampler = h3d.setMaterialSampler
#setMaterialSampler.restype = c_bool
#setMaterialSampler.argtypes = [c_int, c_char_p, c_int]
#__all__.append('setMaterialSampler')

## setPipelineStageActivation - deprecated?
#setPipelineStageActivation = h3d.setPipelineStageActivation
#setPipelineStageActivation.restype = c_bool
#setPipelineStageActivation.argtypes = [c_int, c_char_p, c_bool]
#__all__.append('setPipelineStageActivation')

## winging this one - Alex
_h3dGetPipelineRenderTargetData = h3d.h3dGetPipelineRenderTargetData
_h3dGetPipelineRenderTargetData.restype = c_bool
_h3dGetPipelineRenderTargetData = [c_int, c_char_p, c_int, POINTER(c_int), POINTER(c_int), POINTER(c_int), POINTER(c_float), c_int]
def h3dGetPipelineRenderTargetData(pipelineRes, targetName, bufIndex):
	width = c_int()
	height = c_int()
	compCount = c_int()

	# get values for width, height, compCount to calculate needed buffer size
	_h3dGetPipelineRenderTargetData(pipelineRes, targetName, bufIndex, byref(width), byref(height), byref(compCount), None, 0)

	# get data
	bufferSize = 4 * width * height * compCount
	buffer = (c_float * (bufferSize / 4))()
	_h3dGetPipelineRenderTargetData(pipelineRes, targetName, buffer, None, None, None, buffer)

	return [x for x in buffer] # FIXME really make a normal list of floats out of this array?
__all__.append('h3dGetPipelineRenderTargetData')


h3dGetNodeType = h3d.h3dGetNodeType
h3dGetNodeType.restype = c_int
h3dGetNodeType.argtypes = [c_int]
__all__.append('h3dGetNodeType')


h3dGetNodeParent = h3d.h3dGetNodeParent
h3dGetNodeParent.restype = c_int
h3dGetNodeParent.argtypes = [c_int]
__all__.append('h3dGetNodeParent')


h3dSetNodeParent = h3d.h3dSetNodeParent
h3dSetNodeParent.restype = c_bool
h3dSetNodeParent.argtypes = [c_int, c_int]
__all__.append('h3dSetNodeParent')


h3dGetNodeChild = h3d.h3dGetNodeChild
h3dGetNodeChild.restype = c_int
h3dGetNodeChild.argtypes = [c_int, c_int]
__all__.append('h3dGetNodeChild')


h3dAddNodes = h3d.h3dAddNodes
h3dAddNodes.restype = c_int
h3dAddNodes.argtypes = [c_int, c_int]
__all__.append('h3dAddNodes')


h3dRemoveNode = h3d.h3dRemoveNode
h3dRemoveNode.restype = None
h3dRemoveNode.argtypes = [c_int]
__all__.append('h3dRemoveNode')


h3dSetNodeActivation = h3d.h3dSetNodeActivation
h3dSetNodeActivation.restype = None
h3dSetNodeActivation.argtypes = [c_int, c_bool]
__all__.append('h3dSetNodeActivation')


h3dCheckNodeTransFlag = h3d.h3dCheckNodeTransFlag
h3dCheckNodeTransFlag.restype = c_bool
h3dCheckNodeTransFlag.argtypes = [c_int, c_bool]
__all__.append('h3dCheckNodeTransFlag')

## Needs checking (restype changed from c_bool to void) - Al
_h3dGetNodeTransform = h3d.h3dGetNodeTransform
_h3dGetNodeTransform.restype = None
_h3dGetNodeTransform.argtypes = [c_int,
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		]
def h3dGetNodeTransform(node):
	tx, ty, tz = c_float(), c_float(), c_float()
	rx, ry, rz = c_float(), c_float(), c_float()
	sx, sy, sz = c_float(), c_float(), c_float()

	# FIXME really assert?
	assert(_h3dGetNodeTransform(node,
		byref(tx), byref(ty), byref(tz),
		byref(rx), byref(ry), byref(rz),
		byref(sx), byref(sy), byref(sz)
		))

	return [
			[tx.value, ty.value, tz.value]
			[rx.value, ry.value, rz.value]
			[sx.value, sy.value, sz.value]
			]
__all__.append('h3dGetNodeTransform')

## Needs checking (restype changed from c_bool to void) - Al
_h3dSetNodeTransform = h3d.h3dSetNodeTransform
_h3dSetNodeTransform.restype = None
_h3dSetNodeTransform.argtypes = [c_int,
					c_float, c_float, c_float,
					c_float, c_float, c_float,
					c_float, c_float, c_float,
					]
def h3dSetNodeTransform(node, tx, ty, tz, rx, ry, rz, sx, sy, sz):
	return _h3dSetNodeTransform(node,
			c_float(tx), c_float(ty), c_float(tz),
			c_float(rx), c_float(ry), c_float(rz),
			c_float(sx), c_float(sy), c_float(sz),
			)
__all__.append('h3dSetNodeTransform')

## Needs checking (restype changed from c_bool to void) - WTF what's this * 16 business to the c_floats? I'll just rename the functions like a good script monkey - Al
_h3dGetNodeTransMats = h3d.h3dGetNodeTransMats
_h3dGetNodeTransMats.restype = None
_h3dGetNodeTransMats.argtypes = [c_int, POINTER(c_float * 16), POINTER(c_float * 16)]
def h3dGetNodeTransMats(node):
	t = c_float * 16
	rel = t()
	abs = t()
	# FIXME really assert?
	assert(_h3dGetNodeTransMats(node, byref(rel), byref(abs)))

	a = tuple([x for x in rel])
	b = tuple([x for x in abs])
	return (a, b)
__all__.append('h3dGetNodeTransMats')

## Needs checking (restype changed from c_bool to void)
_h3dSetNodeTransMat = h3d.h3dSetNodeTransMat
_h3dSetNodeTransMat.restype = None
_h3dSetNodeTransMat.argtypes = [c_int, c_float * 16]
def h3dSetNodeTransMat(node, mat):
	t = c_float * 16
	f16 = t()
	for i in range(16):
		f16[i] = mat[i]
	# FIXME really assert?
	assert(_h3dSetNodeTransMat(node, f16))
__all__.append('h3dSetNodeTransMat')


h3dGetNodeParamI = h3d.h3dGetNodeParamI
h3dGetNodeParamI.restype = c_int
h3dGetNodeParamI.argtypes = [c_int, c_int]
__all__.append('h3dGetNodeParamI')

## Needs checking (restype changed from c_bool to void)
h3dSetNodeParamI = h3d.h3dSetNodeParamI
h3dSetNodeParamI.restype = None
h3dSetNodeParamI.argtypes = [c_int, c_int, c_int]
__all__.append('h3dSetNodeParamI')

## Needs checking (compIdx added, needs fixing!)
_h3dGetNodeParamF = h3d.h3dGetNodeParamF
_h3dGetNodeParamF.restype = c_float
_h3dGetNodeParamF.argtypes = [c_int, c_int, c_int]
def h3dGetNodeParamF(node, param, compIdx):
	return _h3dGetNodeParamF(node, param, compIdx).value
__all__.append('h3dGetNodeParamF')


## Needs checking (restype c_bool to None, added compIdx, needs fixing!)
_h3dSetNodeParamF = h3d.h3dSetNodeParamF
_h3dSetNodeParamF.restype = None
_h3dSetNodeParamF.argtypes = [c_int, c_int, c_int, c_float]
def h3dSetNodeParamF(node, param, compIdx, value):
	return _h3dSetNodeParamF(node, param, compIdx, value)
__all__.append('h3dSetNodeParamF')


h3dGetNodeParamStr = h3d.h3dGetNodeParamStr
h3dGetNodeParamStr.restype = c_char_p
h3dGetNodeParamStr.argtypes = [c_int, c_int]
__all__.append('h3dGetNodeParamStr')


h3dSetNodeParamStr = h3d.h3dSetNodeParamStr
h3dSetNodeParamStr.restype = None
h3dSetNodeParamStr.argtypes = [c_int, c_int, c_char_p]
__all__.append('h3dSetNodeParamStr')

# Needs checking (c_bool changed to None)
_h3dGetNodeAABB = h3d.h3dGetNodeAABB
_h3dGetNodeAABB.restype = None
_h3dGetNodeAABB.argtypes = [c_int,
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		]
def h3dGetNodeAABB(node):
	minX, minY, minZ = c_float(), c_float(), c_float()
	maxX, maxY, maxZ = c_float(), c_float(), c_float()

	# FIXME really assert?
	assert(_h3dGetNodeAABB(node,
		byref(minX), byref(minY), byref(minZ),
		byref(maxX), byref(maxY), byref(maxZ),
		))
	return [[minX, minY, minZ], [maxX, maxY, maxZ]]
__all__.append('h3dGetNodeAABB')


h3dFindNodes = h3d.h3dFindNodes
h3dFindNodes.restype = c_int
h3dFindNodes.argtypes = [c_int, c_char_p, c_int]
__all__.append('h3dFindNodes')


h3dGetNodeFindResult = h3d.h3dGetNodeFindResult
h3dGetNodeFindResult.restype = c_int
h3dGetNodeFindResult.argtypes = [c_int]
__all__.append('h3dGetNodeFindResult')


_h3dCastRay = h3d.h3dCastRay
_h3dCastRay.restype = c_int
_h3dCastRay.argtypes = [c_int,
		c_float, c_float, c_float,
		c_float, c_float, c_float,
		c_int]
def h3dCastRay(node, ox, oy, oz, dx, dy, dz, numNearest):
	return _h3dCastRay(node,
			c_float(ox), c_float(oy), c_float(oz),
			c_float(dx), c_float(dy), c_float(dz),
			numNearest)
__all__.append('h3dCastRay')

## Definitely needs checking
_h3dGetCastRayResult = h3d.h3dGetCastRayResult
_h3dGetCastRayResult.restype = c_bool
_h3dGetCastRayResult.argtypes = [c_int, POINTER(c_int), POINTER(c_float), POINTER(c_float * 3)]
def h3dGetCastRayResult(index):
	i = c_int()
	f = c_float()
	f3 = (c_float * 3)()

	# FIXME really assert? throw exception?
	assert(_h3dGetCastRayResult(index, byref(i), byref(f), byref(f3)))

	return [i.value, f.value, tuple([x for x in f3])]
__all__.append('h3dGetCastRayResult')


h3dCheckNodeVisibility = h3d.h3dCheckNodeVisibility
h3dCheckNodeVisibility.restype = c_int
h3dCheckNodeVisibility.argtypes = [c_int, c_int, c_bool, c_bool]
__all__.append('h3dCheckNodeVisibility')


h3dAddGroupNode = h3d.h3dAddGroupNode
h3dAddGroupNode.restype = c_int
h3dAddGroupNode.argtypes = [c_int, c_char_p]
__all__.append('h3dAddGroupNode')


h3dAddModelNode = h3d.h3dAddModelNode
h3dAddModelNode.restype = c_int
h3dAddModelNode.argtypes = [c_int, c_char_p, c_int]
__all__.append('h3dAddModelNode')

## Needs checking, changed c_bool to void/none
h3dSetupModelAnimStage = h3d.h3dSetupModelAnimStage
h3dSetupModelAnimStage.restype = None
h3dSetupModelAnimStage.argtypes = [c_int, c_int, c_int, c_int, c_char_p, c_bool]
__all__.append('h3dSetupModelAnimStage')

## Needs checking
_h3dSetModelAnimParams = h3d.h3dSetModelAnimParams
_h3dSetModelAnimParams.restype = None
_h3dSetModelAnimParams.argtypes = [c_int, c_int, c_float, c_float]
def h3dSetModelAnimParams(modelNode, stage, time, weight):
	return _h3dSetModelAnimParams(modelNode, stage, c_float(time), c_float(weight))
__all__.append('h3dSetModelAnimParams')

# Needs checking, does target need more pointer info?
_h3dSetModelMorpher = h3d.h3dSetModelMorpher
_h3dSetModelMorpher.restype = c_bool
_h3dSetModelMorpher.argtypes = [c_int, c_char_p, c_float]
def h3dSetModelMorpher(modelNode, target, weight):
	return _h3dSetModelMorpher(modelNode, target, c_float(weight))
__all__.append('h3dSetModelMorpher')


h3dAddMeshNode = h3d.h3dAddMeshNode
h3dAddMeshNode.restype = c_int
h3dAddMeshNode.argtypes = [c_int, c_char_p, c_int, c_int, c_int, c_int, c_int]
__all__.append('h3dAddMeshNode')


h3dAddJointNode = h3d.h3dAddJointNode
h3dAddJointNode.restype = c_int
h3dAddJointNode.argtypes = [c_int, c_char_p, c_int]
__all__.append('h3dAddJointNode')


h3dAddLightNode = h3d.h3dAddLightNode
h3dAddLightNode.restype = c_int
h3dAddLightNode.argtypes = [c_int, c_char_p, c_int, c_char_p, c_char_p]
__all__.append('h3dAddLightNode')

## setLightContexts - deprecated?
#setLightContexts = h3d.setLightContexts
#setLightContexts.restype = c_bool
#setLightContexts.argtypes = [c_int, c_char_p, c_char_p]
#__all__.append('setLightContexts')


h3dAddCameraNode = h3d.h3dAddCameraNode
h3dAddCameraNode.restype = c_int
h3dAddCameraNode.argtypes = [c_int, c_char_p, c_int]
__all__.append('h3dAddCameraNode')

## Needs checking - changed c_bool to None
_h3dSetupCameraView = h3d.h3dSetupCameraView
_h3dSetupCameraView.restype = None
_h3dSetupCameraView.argtypes = [c_int, c_float, c_float, c_float, c_float]
def h3dSetupCameraView(cameraNode, fov, aspect, nearDist, farDist):
	return _h3dSetupCameraView(cameraNode, c_float(fov), c_float(aspect), c_float(nearDist), c_float(farDist))
__all__.append('h3dSetupCameraView')


## Needs checking - changed c_bool to None
_h3dGetCameraProjMat = h3d.h3dGetCameraProjMat
_h3dGetCameraProjMat.restype = None
_h3dGetCameraProjMat.argtypes = [c_int, c_float * 16]
def h3dGetCameraProjMat(node, mat):
	t = c_float * 16
	f16 = t()
	for i in range(16):
		f16[i] = mat[i]
	# FIXME really assert?
	assert(_h3dGetCameraProjMat(node, f16))
__all__.append('h3dGetCameraProjMat')


h3dAddEmitterNode = h3d.h3dAddEmitterNode
h3dAddEmitterNode.restype = c_int
h3dAddEmitterNode.argtypes = [c_int, c_char_p, c_int, c_int, c_int, c_int]
__all__.append('h3dAddEmitterNode')


## Needs checking - changed c_bool to None
_h3dAdvanceEmitterTime = h3d.h3dAdvanceEmitterTime
_h3dAdvanceEmitterTime.restype = None
_h3dAdvanceEmitterTime.argtypes = [c_int, c_float]
def h3dAdvanceEmitterTime(emitterNode, timeDelta):
	return _h3dAdvanceEmitterTime(emitterNode, c_float(timeDelta))
__all__.append('h3dAdvanceEmitterTime')


h3dHasEmitterFinished = h3d.h3dHasEmitterFinished
h3dHasEmitterFinished.restype = c_bool
h3dHasEmitterFinished.argtypes = [c_int]
__all__.append('h3dHasEmitterFinished')


