# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
# --------------------------------------
# Copyright (C) 2006-2009 Nicolas Schulz
#			   2008-2009 Florian Noeding (Python wrapper)
#
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# *************************************************************************************************

from ctypes import *

if not 'c_bool' in globals():
	c_bool = c_int


__all__ = []



try:
	h3d = cdll.LoadLibrary('libHorde3D.so')
except OSError:
	h3d = cdll.LoadLibrary('Horde3D.dll')

if hasattr(h3d, 'h3dGetActiveListener'):
	RST_SoundResource = 200
	__all__.append('RST_SoundResource')
	SNT_ListenerNode = 201
	__all__.append('SNT_ListenerNode')
	SNT_SoundNode = 202
	__all__.append('SNT_SoundNode')


	class H3DSoundResParams(object):
		SamplingRate = 20000
		BitDepth = 20001
		Channels = 20002
		BitRate = 20003
		Runtime = 20004
	__all__.append('H3DSoundResParams')


	class H3DListenerNodeParams(object):
		MasterGain = 20000
		DopplerFactor = 20001
		SpeedOfSound = 20002
	__all__.append('H3DListenerNodeParams')


	class H3DSoundNodeParams(object):
		SoundRes = 20000
		Gain = 20001
		Pitch = 20002
		Offset = 20003
		Loop = 20004
		MaxDistance = 20005
		RolloffFactor = 20006
		ReferenceDistance = 20007
		MinGain = 20008
		MaxGain = 20009
	__all__.append('H3DSoundNodeParams')


	class H3DDistanceModels(object):
		None_ = 0
		InverseDistance = 1
		InverseDistanceClamped = 2
		LinearDistance = 3
		LinearDistanceClamped = 4
		ExponentDistance = 5
		ExponentDistanceClamped = 6
	__all__.append('H3DDistanceModels')


	h3dOpenDevice = h3d.h3dOpenDevice
	h3dOpenDevice.restype = c_bool
	h3dOpenDevice.argtypes = [c_char_p]
	__all__.append('h3dOpenDevice')


	h3dCloseDevice = h3d.h3dCloseDevice
	h3dCloseDevice.restype = None
	h3dCloseDevice.argtypes = []
	__all__.append('h3dCloseDevice')


	h3dGetOpenDevice = h3d.h3dGetOpenDevice
	h3dGetOpenDevice.restype = c_char_p
	h3dGetOpenDevice.argtypes = []
	__all__.append('h3dGetOpenDevice')


	h3dQueryDevice = h3d.h3dQueryDevice
	h3dQueryDevice.restype = c_char_p
	h3dQueryDevice.argtypes = [c_int]
	__all__.append('h3dQueryDevice')

## DLL H3DDistanceModels::List h3dGetDistanceModel(); - needs special binding?
	h3dGetDistanceModel = h3d.h3dGetDistanceModel
	h3dGetDistanceModel.restype = c_int
	h3dGetDistanceModel.argtypes = []
	__all__.append('h3dGetDistanceModel')


	h3dSetDistanceModel = h3d.h3dSetDistanceModel
	h3dSetDistanceModel.restype = None
	h3dSetDistanceModel.argtypes = [c_int]
	__all__.append('h3dSetDistanceModel')


	h3dAddListenerNode = h3d.h3dAddListenerNode
	h3dAddListenerNode.restype = c_int
	h3dAddListenerNode.argtypes = [c_int, c_char_p]
	__all__.append('h3dAddListenerNode')


	h3dGetActiveListener = h3d.h3dGetActiveListener
	h3dGetActiveListener.restype = c_int
	h3dGetActiveListener.argtypes = []
	__all__.append('h3dGetActiveListener')


	h3dSetActiveListener = h3d.h3dSetActiveListener
	h3dSetActiveListener.restype = None
	h3dSetActiveListener.argtypes = [c_int]
	__all__.append('h3dSetActiveListener')


	h3dAddSoundNode = h3d.h3dAddSoundNode
	h3dAddSoundNode.restype = c_int
	h3dAddSoundNode.argtypes = [c_int, c_char_p, c_int]
	__all__.append('h3dAddSoundNode')


	h3dIsSoundPlaying = h3d.h3dIsSoundPlaying
	h3dIsSoundPlaying.restype = c_bool
	h3dIsSoundPlaying.argtypes = [c_int]
	__all__.append('h3dIsSoundPlaying')


	h3dPlaySound = h3d.h3dPlaySound
	h3dPlaySound.restype = None
	h3dPlaySound.argtypes = [c_int]
	__all__.append('h3dPlaySound')


	h3dPauseSound = h3d.h3dPauseSound
	h3dPauseSound.restype = None
	h3dPauseSound.argtypes = [c_int]
	__all__.append('h3dPauseSound')


	h3dRewindSound = h3d.h3dRewindSound
	h3dRewindSound.restype = None
	h3dRewindSound.argtypes = [c_int]


