# *************************************************************************************************
#
# Horde3D
#   Next-Generation Graphics Engine
# --------------------------------------
# Copyright (C) 2006-2009 Nicolas Schulz
#               2008-2009 Florian Noeding (Python wrapper)
#               2009 Alex Fuller (Python wrapper update)
#
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# *************************************************************************************************

from ctypes import *

if not 'c_bool' in globals():
	c_bool = c_int


__all__ = []



try:
	utils = cdll.LoadLibrary('libHorde3DUtils.so')
except OSError:
	utils = cdll.LoadLibrary('Horde3DUtils.dll')


h3dutDumpMessages = utils.h3dutDumpMessages
h3dutDumpMessages.restype = c_bool
h3dutDumpMessages.argtypes = []
__all__.append('h3dutDumpMessages')


h3dutGetResourcePath = utils.h3dutGetResourcePath
h3dutGetResourcePath.restype = c_char_p
h3dutGetResourcePath.argtypes = [c_int]
__all__.append('h3dutGetResourcePath')


h3dutSetResourcePath = utils.h3dutSetResourcePath
h3dutSetResourcePath.restype = None
h3dutSetResourcePath.argtypes = [c_int, c_char_p]
__all__.append('h3dutSetResourcePath')


h3dutLoadResourcesFromDisk = utils.h3dutLoadResourcesFromDisk
h3dutLoadResourcesFromDisk.restype = c_bool
h3dutLoadResourcesFromDisk.argtypes = [c_char_p]
__all__.append('h3dutLoadResourcesFromDisk')


_h3dutPickRay = utils.h3dutPickRay
_h3dutPickRay.restype = None
_h3dutPickRay.argtypes = [
		c_int,
		c_float, c_float,
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		POINTER(c_float), POINTER(c_float), POINTER(c_float),
		]
def h3dutPickRay(cameraNode, nwx, nwy):
	ox, oy, oz = c_float(), c_float(), c_float()
	dx, dy, dz = c_float(), c_float(), c_float()
	_h3dutPickRay(cameraNode,
			c_float(nwx), c_float(nwy),
			byref(ox), byref(oy), byref(oz),
			byref(dx), byref(dy), byref(dz),
			)
	return [[ox.value, oy.value, oz.value], [dx.value, dy.value, dz.value]]
__all__.append('h3dutPickRay')


_h3dutPickNode = utils.h3dutPickNode
_h3dutPickNode.restype = c_int
_h3dutPickNode.argtypes = [c_int, c_float, c_float]
def h3dutPickNode(cameraNode, nwx, nwy):
	return _h3dutPickNode(cameraNode, c_float(nwx), c_float(nwy))
__all__.append('h3dutPickNode')


_h3dutShowText = utils.h3dutShowText
_h3dutShowText.restype = None
_h3dutShowText.argtypes = [c_char_p, c_float, c_float, c_float, c_float, c_float, c_float, c_int, c_int]
def h3dutShowText(text, x, y, size, r, g, b, fontMaterialRes, layer):
	return _h3dutShowText(text, c_float(x), c_float(y), c_float(size), c_float(r), c_float(g), c_float(b), fontMaterialRes, layer)
__all__.append('h3dutShowText')


_h3dutShowFrameStats = utils.h3dutShowFrameStats
_h3dutShowFrameStats.restype = None
_h3dutShowFrameStats.argtypes = [c_int, c_int, c_int]
def h3dutShowFrameStats(fontMaterialRes, panelMaterialRes, mode):
	return _h3dutShowFrameStats(fontMaterialRes, panelMaterialRes, mode)
__all__.append('h3dutShowFrameStats')
